@@ -1,11 +1,10 @@
-import yaml
 import traceback
 import datetime
 import os
 import time
 import socket
 from hashlib import sha224
-from collections import OrderedDict, defaultdict
+from collections import defaultdict
 import glob
 import logging
 import io
@@ -16,12 +15,12 @@ import dqueue.core
 import dqueue.app
 import dqueue.tools as tools
 
-import peewee
+import peewee # type: ignore
 import json
 
 from flask import Flask
 from flask import render_template,make_response,request,jsonify
-from flasgger import Swagger, SwaggerView, Schema, fields
+from flasgger import Swagger, SwaggerView, Schema, fields # type: ignore
 
 decoded_entries={} # type: ignore
 
@@ -58,6 +57,7 @@ swagger = Swagger(app, template=template)
 
 print("setting up app", app, id(app))
 
+logging.basicConfig(level=logging.DEBUG)
 logger=logging.getLogger(__name__)
 
 
@@ -75,6 +75,16 @@ class Task(Schema):
 class TaskList(Schema):
     tasks = fields.Nested(Task, many=True)
 
+class LogEntry(Schema):
+    message = fields.Str()
+
+class QueueLogEntry(Schema):
+    state = fields.Str()
+    task_key = fields.Str()
+
+class ViewLog(Schema):
+    event_log = fields.Nested(LogEntry, many=True)
+
 class QueueList(Schema):
     queues = fields.Nested(fields.Str(), many=True)
 
@@ -86,6 +96,12 @@ class Status(Schema):
 class TaskListView(SwaggerView):
     operationId = "listTasks"
     parameters = [
+        {
+            "name": "queue",
+            "in": "query",
+            "type": "string",
+            "required": False,
+        },
         {
             "name": "state",
             "in": "query",
@@ -107,8 +123,16 @@ class TaskListView(SwaggerView):
         get list of tasks
         """
 
+        queue = dqueue.core.Queue(request.args.get('queue', 'default'))
+
+        tasks = [e for e in tools.list_tasks(include_task_data=True)]
+
+        #tasks = queue.list_tasks(decode=True) #state=state)
+
+        logger.debug("list tasks returns %s, tasks", tasks)
+
         return jsonify(
-                tasks=[e for e in tools.list_tasks(include_task_data=True)]
+                tasks=tasks
             )
 
 app.add_url_rule(
@@ -123,7 +147,7 @@ class WorkerOffer(SwaggerView):
     parameters = [
                 {
                     'name': 'worker_id',
-                    'in': 'query',
+                    'in': 'path',
                     'required': True,
                     'type': 'string',
                 },
@@ -151,8 +175,8 @@ class WorkerOffer(SwaggerView):
                 }
         }
 
-    def get(self):
-        queue = dqueue.core.Queue(request.args.get('queue', 'default'))
+    def get(self, worker_id):
+        queue = dqueue.core.Queue(request.args.get('queue', 'default'), worker_id=worker_id)
 
         try:
             task = queue.get()
@@ -170,7 +194,7 @@ class WorkerOffer(SwaggerView):
 
 
 app.add_url_rule(
-         '/worker/offer',
+          '/worker/<string:worker_id>/offer',
           view_func=WorkerOffer.as_view('worker_offer_task'),
           methods=['GET']
 )
@@ -220,7 +244,7 @@ class WorkerAnswer(SwaggerView):
         logger.debug("setting current task in %s to %s", queue, task_dict)
 
         queue.state = "done"
-        queue.current_task = dqueue.core.Task.from_entry(task_dict)
+        queue.current_task = dqueue.core.Task.from_task_dict(task_dict)
         queue.current_task_stored_key = queue.current_task.key
         task = queue.current_task
 
@@ -238,7 +262,114 @@ app.add_url_rule(
 )
 
 
-class TaskLog(SwaggerView):
+class TryAllLocked(SwaggerView):
+    operationId = "try_all_locked"
+
+    # locally or remotely?
+
+    parameters = [
+                {
+                    'name': 'worker_id',
+                    'in': 'path',
+                    'required': True,
+                    'type': 'string',
+                },
+                {
+                    'name': 'queue',
+                    'in': 'query',
+                    'required': False,
+                    'type': 'string',
+                },
+                {
+                    'name': 'token',
+                    'in': 'query',
+                    'required': True,
+                    'type': 'string',
+                },
+            ]
+
+    responses = {
+            200: {
+                    'description': 'unlocked!',
+                },
+            204: {
+                    'description': 'problem: no tasks can be offered',
+                }
+        }
+
+    def get(self, worker_id):
+        queue = dqueue.core.Queue(request.args.get('queue', 'default'), worker_id=worker_id)
+
+        r = queue.try_all_locked()
+
+        logger.info("unlocked: %d", len(r))
+
+        return jsonify(tasks=r)
+
+
+app.add_url_rule(
+          '/tasks/<string:worker_id>/try_all_locked',
+          view_func=TryAllLocked.as_view('tasks_try_all_locked'),
+          methods=['GET']
+)
+
+class TaskViewLog(SwaggerView):
+    operationId = "view_log"
+
+    parameters = [
+                {
+                    'name': 'task_key',
+                    'in': 'query',
+                    'required': False,
+                    'type': 'string',
+                },
+                {
+                    'name': 'since',
+                    'in': 'query',
+                    'required': False,
+                    'type': 'number',
+                },
+                {
+                    'name': 'token',
+                    'in': 'query',
+                    'required': True,
+                    'type': 'string',
+                },
+            ]
+
+    responses = {
+            200: {
+                    'description': 'task log dict',
+                    'schema': ViewLog,
+                }
+        }
+
+    def get(self):
+        task_key = request.args.get('task_key', None)
+        if task_key == "":
+            task_key = None
+
+        since = request.args.get('since', 0, type=int)
+
+        queue = dqueue.core.Queue()
+
+        r = queue.view_log(task_key=task_key, since=since)
+
+        logger.info("view_log api returns %d entris", len(r))
+        #for e in r:
+        #    logger.info("view_log api returns entry: %s", e)
+
+        return jsonify(
+                    event_log=r,
+                )
+
+app.add_url_rule(
+     '/task/view_log',
+      view_func=TaskViewLog.as_view('task_view_log'),
+      methods=['GET']
+)
+
+class TaskLogView(SwaggerView):
     operationId = "logTask"
 
     parameters = [
@@ -257,7 +388,7 @@ class TaskLog(SwaggerView):
                 {
                     'name': 'queue',
                     'in': 'query',
-                    'required': True,
+                    'required': False,
                     'type': 'string',
                 },
                 {
@@ -287,25 +418,92 @@ class TaskLog(SwaggerView):
         }
 
     def post(self):
-        message = request.args.get('message')
-        queue = request.args.get('queue')
+        message = request.args.get('message', 'no message')
+        queue = request.args.get('queue', 'default')
         task_key = request.args.get('task_key')
         worker_id = request.args.get('worker_id')
         state = request.args.get('state')
 
         queue = dqueue.core.Queue(worker_id=worker_id, queue=queue)
 
-        logger.info("task log worker_id %s task_key %s", worker_id, task_key)
+        logger.info("log_task worker_id %s task_key %s message %s", worker_id, task_key, message)
 
-        queue.log_task(message, task_key=task_key, state=state)
+        queue.log_task(message=message, task_key=task_key, state=state)
+
+        logger.debug("task log: %s", queue.view_log())
+        assert len(queue.view_log()) > 0
 
         return jsonify(
                     message
                 )
 
 app.add_url_rule(
-     '/worker/tasklog',
-      view_func=TaskLog.as_view('worker_task_log'),
+     '/worker/log',
+      view_func=TaskLogView.as_view('worker_log'),
+      methods=['POST']
+)
+
+class QueueLogView(SwaggerView):
+    operationId = "logQueue"
+
+    parameters = [
+                {
+                    'name': 'message',
+                    'in': 'query',
+                    'required': True,
+                    'type': 'string',
+                },
+                {
+                    'name': 'spent_s',
+                    'in': 'query',
+                    'required': True,
+                    'type': 'number',
+                },
+                {
+                    'name': 'queue',
+                    'in': 'query',
+                    'required': False,
+                    'type': 'string',
+                },
+                {
+                    'name': 'worker_id',
+                    'in': 'query',
+                    'required': True,
+                    'type': 'string',
+                },
+                {
+                    'name': 'token',
+                    'in': 'query',
+                    'required': False,
+                    'type': 'string',
+                },
+            ]
+
+    responses = {
+            200: {
+                    'description': 'task dict',
+                }
+        }
+
+    def post(self):
+        message = request.args.get('message')
+        spent_s = request.args.get('spent_s', type=float)
+        worker_id = request.args.get('worker_id')
+        queue = request.args.get('queue', 'default')
+
+        queue = dqueue.core.Queue(worker_id=worker_id, queue=queue)
+
+        logger.info("log_queue worker_id %s message %s spent %s", worker_id, message, spent_s)
+
+        logger.info("log queue returns: %s", queue.log_queue(message, spent_s, worker_id=worker_id))
+
+        return jsonify(
+                    message
+                )
+
+app.add_url_rule(
+     '/worker/queuelog',
+      view_func=QueueLogView.as_view('worker_queue_log'),
       methods=['POST']
 )
 
@@ -368,7 +566,9 @@ app.add_url_rule(
       methods=['POST']
 )
 
-class TaskView(SwaggerView):
+class TaskInfoView(SwaggerView):
+    operationId = "task_info"
+
     parameters = [
                 {
                     'name': 'task_key',
@@ -386,13 +586,24 @@ class TaskView(SwaggerView):
         }
 
     def get(self, task_key):
-        info = tools.task_info(task_key)
-        logger.warning("requested task_key %s %s", task_key, info)
+        queue = dqueue.core.Queue()
+
+        task_dict = queue.task_by_key(task_key)
+        logger.warning("requested task_key %s %s", task_key, task_dict)
+
+        if task_dict is None:
+            return jsonify()
+
         return jsonify(
-                task_key=task_key,
-                task_info=info,
+                **task_dict
             )
 
+app.add_url_rule(
+         '/task/view/<task_key>',
+          view_func=TaskInfoView.as_view('view_task_info'),
+          methods=['GET']
+)
+
 @app.route("/tasks/resubmit/<string:scope>/<string:selector>")
 def tasks_resubmit(scope, selector):
     """
@@ -491,10 +702,5 @@ def handle(error):
 
     traceback.print_exc()
 
-app.add_url_rule(
-         '/task/view/<task_key>',
-          view_func=TaskView.as_view('api_task'),
-          methods=['GET']
-)
 
 print("app added rules", id(app))
