@@ -1,6 +1,6 @@
-import yaml
 import datetime
 import os
+import json
 import time
 import socket
 from hashlib import sha224
@@ -25,10 +25,10 @@ try:
 except ImportError:
     import urllib.parse as urlparse# type: ignore
 
-from typing import NewType, Dict
+from typing import NewType, Dict, Union, List
 
-TaskDict = NewType('TaskDict', Dict)
-TaskData = NewType('TaskData', Dict)
+import dqueue.typing as types
+from dqueue.entry import decode_entry_data
 
 import pymysql
 import peewee # type: ignore
@@ -38,12 +38,24 @@ from playhouse.shortcuts import model_to_dict, dict_to_model # type: ignore
 sleep_multiplier = 1
 n_failed_retries = int(os.environ.get('DQUEUE_FAILED_N_RETRY','20'))
 
-logger=logging.getLogger(__name__)
-logger.setLevel(logging.INFO)
-handler=logging.StreamHandler()
-logger.addHandler(handler)
-formatter = logging.Formatter('%(asctime)s %(levelname)8s %(name)s | %(message)s')
-handler.setFormatter(formatter)
+
+def get_logger(name):
+    level = getattr(logging, os.environ.get('DQUEUE_LOG_LEVEL', 'INFO'))
+
+    logging.basicConfig(level=level)
+
+    logger=logging.getLogger(name)
+    logger.setLevel(level)
+
+    handler=logging.StreamHandler()
+    logger.addHandler(handler)
+
+    formatter = logging.Formatter('%(asctime)s %(levelname)8s %(name)s | %(message)s')
+    handler.setFormatter(formatter)
+
+    return logger
+
+logger = get_logger(__name__)
 
 def log(*args,**kwargs):
     severity=kwargs.get('severity','warning').upper()
@@ -72,12 +84,15 @@ except Exception as e:
     
 
 class TaskEntry(peewee.Model):
+    database = None
+
     queue = peewee.CharField(default="default")
 
     key = peewee.CharField(primary_key=True)
     state = peewee.CharField()
     worker_id = peewee.CharField()
-    entry = peewee.TextField()
+
+    task_dict_string = peewee.TextField()
 
     created = peewee.DateTimeField()
     modified = peewee.DateTimeField()
@@ -85,21 +100,25 @@ class TaskEntry(peewee.Model):
     class Meta:
         database = db
 
-class TaskHistory(peewee.Model):
+
+class EventLog(peewee.Model):
     queue = peewee.CharField(default="default")
 
-    key = peewee.CharField()
-    state = peewee.CharField()
+    task_key = peewee.CharField(default="unset")
+    state = peewee.CharField(default="unset")
+
     worker_id = peewee.CharField()
 
     timestamp = peewee.DateTimeField()
     message = peewee.CharField()
+    
+    spent_s = peewee.FloatField(default=0)
 
     class Meta:
         database = db
 
 try:
-    db.create_tables([TaskEntry,TaskHistory])
+    db.create_tables([TaskEntry, EventLog])
     has_mysql = True
 except peewee.OperationalError:
     has_mysql = False
@@ -107,11 +126,19 @@ except Exception:
     has_mysql = False
 
 class Task:
-    def __init__(self,task_data,execution_info=None, submission_data=None, depends_on=None):
-        self.task_data=task_data
-        self.submission_info=self.construct_submission_info()
-        self.depends_on=depends_on
+    reference_task = False
 
+    def __init__(self,task_data,execution_info=None, submission_data=None, depends_on=None):
+        if 'task_key' in task_data:
+            self._key = task_data['task_key']
+            self.task_data = None
+            self.reference_task = True
+            logger.debug('creating referenced task %s from %s', self, task_data)
+        else:
+            self.task_data = task_data
+            self.depends_on = depends_on
+
+        self.submission_info=self.construct_submission_info()
         if submission_data is not None:
             self.submission_info.update(submission_data)
 
@@ -135,19 +162,37 @@ class Task:
                 depends_on=self.depends_on,
             )
 
-    def serialize(self):
-        return yaml.dump(self.as_dict,
-            default_flow_style=False, default_style=''
+    @property
+    def reference_dict(self):
+        return dict(task_key=self.key)
+
+    def serialize(self) -> str:
+        return json.dumps(normalize_nested_dict(self.as_dict),
+            sort_keys=True,
         )
 
 
     @classmethod
-    def from_entry(cls,entry):
+    def from_task_dict(cls, entry: Union[str, Dict]):
         if isinstance(entry, str):
-            task_dict = yaml.load(io.StringIO(entry) , Loader=yaml.Loader )
+            try:
+                task_dict = json.loads(entry)
+            except Exception as e:
+                logger.error("problem decoding json from task entry: %s", e)
+                for i, e_l in enumerate(entry.splitlines()):
+                    print(f"problematic json: {i:5d}", e_l)
+                open("/tmp/problematic_entry.json", "wt").write(entry)
+                raise
         else:
             task_dict = entry
 
+        if 'task_data' not in task_dict:
+            logger.error("failed to build Task from entry %s", entry)
+            logger.error("problematic task_dict %s", task_dict)
+            logger.error("problematic task_dict keys %s", list(task_dict.keys()))
+            raise RuntimeError(f"failed to build Task from entry {entry}")
+
+
         print(task_dict['task_data'].keys())
 
         self=cls(task_dict['task_data'])
@@ -164,14 +209,17 @@ class Task:
         return self.get_key(True)
 
     def get_key(self,key=True):
+        if hasattr(self, '_key'):
+            return self._key
+
         components = []
 
-        task_data_string = yaml.dump(order_task_data(self.task_data), encoding='utf-8')
+        task_data_string = json.dumps(order_nested_dict(self.task_data), sort_keys=True)
 
         logger.debug("task data: %s", self.task_data)
         logger.debug("task data string: %s", task_data_string)
 
-        components.append(sha224(task_data_string).hexdigest()[:8])
+        components.append(sha224(task_data_string.encode()).hexdigest()[:8])
         #log("encoding: "+repr(components))
         #log(task_data_string)
         #log("encoding: "+repr(components),severity="debug")
@@ -181,16 +229,17 @@ class Task:
             components.append("%.14lg"%self.submission_info['time'])
             components.append(self.submission_info['utc'])
 
-            components.append(sha224(str(OrderedDict(sorted(self.submission_info.items()))).encode('utf-8')).hexdigest()[:8])
+            s = json.dumps(order_nested_dict(self.submission_info), sort_keys=True)
+            components.append(sha224(s.encode()).hexdigest()[:8])
 
         key = "_".join(components)
 
-        logger.warning("generating key %s from %s", key, task_data_string)
+        logger.warning("generating key %s", key)
 
         return key
 
     def __repr__(self):
-        return "[{}: {}]".format(self.__class__.__name__,self.task_data)
+        return "[{}: {}: {}]".format(self.__class__.__name__, self.key, self.task_data)
 
     def filename_instance(self):
         return "unset"
@@ -209,13 +258,27 @@ def list_queues(pattern=None):
         return [ Queue(task_entry.queue) for task_entry in 
                 TaskEntry.select(TaskEntry.queue).where(TaskEntry.queue % pattern).distinct(TaskEntry.queue) ]
 
+def normalize_nested_dict(d):
+    if isinstance(d, dict) or isinstance(d, OrderedDict) or isinstance(d, defaultdict):
+        return {
+                k:normalize_nested_dict(v) for k, v in sorted(d.items())
+            }
 
-def order_task_data(d):
+    if isinstance(d, tuple) or isinstance(d, list):
+        return [ normalize_nested_dict(i) for i in d ]
+
+    return d
+
+
+def order_nested_dict(d):
     if isinstance(d, dict) or isinstance(d, OrderedDict) or isinstance(d, defaultdict):
         return OrderedDict({
-                k:order_task_data(v) for k, v in sorted(d.items())
+                k:order_nested_dict(v) for k, v in sorted(d.items())
             })
 
+    if isinstance(d, tuple) or isinstance(d, list):
+        return [order_nested_dict(i) for i in d]
+
     return d
 
 
@@ -236,20 +299,20 @@ class Queue:
         self.current_task=None
         self.current_task_status=None
 
-    def find_task_instances(self,task,klist=None):
+    def find_task_instances(self, task: Task, klist: Union[list, None]=None) -> List[types.TaskEntry]:
         ""
         log("find_task_instances for",task.key,"in",self.queue)
+
         if klist is None:
             klist=["waiting", "running", "done", "failed", "locked"]
 
         instances_for_key=[
-                dict(task_entry=task_entry) for task_entry in TaskEntry.select().where(TaskEntry.state << klist, TaskEntry.key==task.key, TaskEntry.queue==self.queue)
+                model_to_dict(task_entry) for task_entry in TaskEntry.select().where(TaskEntry.state << klist, TaskEntry.key==task.key, TaskEntry.queue==self.queue)
             ]
 
         log("found task instances for",task.key,"N == ",len(instances_for_key))
         for i in instances_for_key:
-            i['state'] = i['task_entry'].state
-            log(i['state'], i['task_entry'])
+            log(i['state'], i['task_dict_string'])
 
         return instances_for_key
     
@@ -286,60 +349,81 @@ class Queue:
         log("task still locked", task.key)
         return dict(state="locked",key=task.key)
     
-    
-    def select_task_entry(self,key):
+    def task_by_key(self, key: str, decode: bool=False) -> Union[types.TaskDict, None]:
         ""
 
         r=TaskEntry.select().where(
                          TaskEntry.key==key,
-                        ).execute()
-        assert len(r)==1
-        return r[0]
+                        ).execute(database=None)
+
+        if len(r) > 1:
+            raise RuntimeError(f"found multiple entries for key {key}: suspecting database inconsistency!")
+        
+        if len(r) == 0:
+            return None
+
+        r = model_to_dict(r[0])
+
+        if decode:
+            decode_entry_data(r)
+
+        return r
+
+
     
-    def put(self, task_data: TaskData, submission_data=None, depends_on=None) -> TaskDict:
+    def put(self, task_data: types.TaskData, submission_data=None, depends_on=None) -> Union[types.TaskEntry, None]:
         logger.info("putting in queue task_data %s", task_data)
 
-        assert depends_on is None or type(depends_on) in [list, tuple]
+        assert depends_on is None or type(depends_on) in [list, tuple] # runtime typing!
 
-        task=Task(task_data,submission_data=submission_data,depends_on=depends_on)
+        if depends_on is not None:
+            depends_on = [ Task(dep).reference_dict for dep in depends_on ]
+
+        task=Task(task_data, submission_data=submission_data, depends_on=depends_on)
+
+        instances_for_key = None # type: Union[List[types.TaskEntry], None]
 
         ntry_race=10
         retry_sleep_race=2
         while ntry_race>0:
-            instances_for_key=self.find_task_instances(task)
-            if len(instances_for_key)<=1:
-                break
+            instances_for_key = self.find_task_instances(task) 
             log("found instances for key:",instances_for_key)
-            log("found unexpected number of instances for key:",len(instances_for_key))
+
+            if instances_for_key is not None:
+                if len(instances_for_key)<=1:
+                    break
+
+                logger.error("found unexpected number of instances for key: %d", len(instances_for_key))
+            else:
+                logger.error("instances_for_key is None!")
+
             log("sleeping for",retry_sleep_race,"attempt",ntry_race)
             time.sleep(retry_sleep_race)
             ntry_race-=1
 
-        if len(instances_for_key)>1:
+        if instances_for_key is None or len(instances_for_key)>1:
             raise Exception("probably race condition, multiple task instances:",instances_for_key)
 
-
+        instance_for_key = None # type: Union[types.TaskEntry, None]
         if len(instances_for_key) == 1:
-            instance_for_key=instances_for_key[0]
-        else:
-            instance_for_key=None
+            instance_for_key = instances_for_key[0]
 
         if instance_for_key is not None:
-            log("found existing instance(s) for this key, no need to put:",instances_for_key)
-            self.log_task("task already found",task,instance_for_key['state'])
-            d = model_to_dict(instance_for_key['task_entry'])
-            log("task entry:", d)
+            log("found existing instance(s) for this key, no need to put:", instances_for_key)
+            self.log_task("task already found", task,instance_for_key['state'])
+            d = instance_for_key
+            logger.debug("task entry: %s", d)
             return d
 
         if depends_on is None:
-            self.insert_task_entry(task,"waiting")
+            self.insert_task_entry(task, "waiting")
             log("task inserted as waiting")
         else:
-            self.insert_task_entry(task,"locked")
+            self.insert_task_entry(task, "locked")
             log("task inserted as locked")
 
-        instance_for_key=self.find_task_instances(task)[0]
-        recovered_task=Task.from_entry(instance_for_key['task_entry'].entry)
+        instance_for_key = self.find_task_instances(task)[0]
+        recovered_task = Task.from_task_dict(instance_for_key['task_dict_string']) # type: ignore
 
         if recovered_task.key != task.key:
             log("inconsitent storage:")
@@ -357,22 +441,17 @@ class Queue:
 
             raise Exception("Inconsistent storage")
 
-        log("successfully put in queue:",instance_for_key['task_entry'].entry)
+        logger.debug("successfully put in queue: %s",instance_for_key['task_dict_string'])
 
-        #return dict(state="submitted", task_entry=instance_for_key['task_entry'].entry)
-        d = model_to_dict(instance_for_key['task_entry'])
-        d['state'] = 'submitted'
-        return d
+        
+        instance_for_key['state'] = 'submitted'
+        return instance_for_key
 
     def get(self):
         ""
         if self.current_task is not None:
             raise CurrentTaskUnfinished(self.current_task)
 
-       # tasks=self.list("waiting")
-       # task=tasks[-1]
-       # self.current_task = Task.from_entry(task['task_entry'].entry)
-
     
         r=TaskEntry.update({
                         TaskEntry.state:"running",
@@ -380,18 +459,18 @@ class Queue:
                         TaskEntry.modified:datetime.datetime.now(),
                     })\
                     .order_by(TaskEntry.created)\
-                    .where( (TaskEntry.state=="waiting") & (TaskEntry.queue==self.queue) ).limit(1).execute()
+                    .where( (TaskEntry.state=="waiting") & (TaskEntry.queue==self.queue) ).limit(1).execute(database=None)
 
         if r==0:
-            self.try_all_locked()
+            #self.try_all_locked()
             raise Empty()
 
-        entries=TaskEntry.select().where(TaskEntry.worker_id==self.worker_id,TaskEntry.state=="running").order_by(TaskEntry.modified.desc()).limit(1).execute()
+        entries=TaskEntry.select().where(TaskEntry.worker_id==self.worker_id,TaskEntry.state=="running").order_by(TaskEntry.modified.desc()).limit(1).execute(database=None)
         if len(entries)>1:
             raise Exception(f"several tasks ({len(entries)}) are running for this worker: impossible!")
 
         entry=entries[0]
-        self.current_task=Task.from_entry(entry.entry)
+        self.current_task=Task.from_task_dict(entry.task_dict_string)
         self.current_task_stored_key=self.current_task.key
 
         if self.current_task.key != entry.key:
@@ -424,18 +503,22 @@ class Queue:
         log('task',self.current_task.submission_info)
 
         return self.current_task
-
+    
     def clear_task_history(self):
+        # compatibility
+        return self.clear_event_log()
+
+    def clear_event_log(self):
         ""
         print('this is very descructive')
-        TaskHistory.delete().execute()
+        EventLog.delete().execute(database=None)
 
     def move_task(self, fromk, tok, task, update_entry=False):
         ""
 
         extra = {}
         if update_entry:
-            extra = {TaskEntry.entry: self.current_task.serialize()}
+            extra = {TaskEntry.task_dict_string: self.current_task.serialize()}
 
         r=TaskEntry.update({
                         TaskEntry.state:tok,
@@ -443,24 +526,38 @@ class Queue:
                         TaskEntry.modified:datetime.datetime.now(),
                         **extra
                     })\
-                    .where(TaskEntry.state==fromk, TaskEntry.key==task.key).execute()
+                    .where(TaskEntry.state==fromk, TaskEntry.key==task.key).execute(database=None)
 
     def purge(self):
         ""
-        nentries=TaskEntry.delete().execute()
+        nentries=TaskEntry.delete().execute(database=None)
         log("deleted %i"%nentries)
 
         return nentries
 
     
-    def try_all_locked(self):
+    def try_all_locked(self, unlock_max = 10):
         ""
         r=[]
-        for task_key in self.list("locked"):
-            task_entry=self.select_task_entry(task_key)
-            log("trying to unlock", task_entry.key)
-            #log("trying to unlock", task_key,task_entry,task_entry.key,task_entry.entry)
-            r.append(self.try_to_unlock(Task.from_entry(task_entry.entry)))
+
+        n_unlocked = 0
+
+        locked_tasks = self.list_tasks(state="locked")
+
+        logger.info("found %d locked tasks", len(locked_tasks))
+
+        for task_key in locked_tasks:
+            task_entry = self.task_by_key(task_key)
+            logger.info("trying to unlock %s", task_entry['key'])
+
+            r.append(self.try_to_unlock(Task.from_task_dict(task_entry['task_dict_string'])))
+
+            if r[-1]['state'] != "locked":
+                n_unlocked += 1
+
+            if n_unlocked >= unlock_max:
+                break
+
         return r
     
     def remember(self,task_data,submission_data=None):
@@ -473,12 +570,12 @@ class Queue:
     def insert_task_entry(self,task,state):
         self.log_task("task created",task,state)
 
-        log("to insert_task_entry: ", dict(
+        log("to insert_task entry: ", dict(
              queue=self.queue,
              key=task.key,
              state=state,
              worker_id=self.worker_id,
-             entry=task.serialize(),
+             task_dict_string=task.serialize(),
              created=datetime.datetime.now(),
              modified=datetime.datetime.now(),
         ))
@@ -489,10 +586,10 @@ class Queue:
                              key=task.key,
                              state=state,
                              worker_id=self.worker_id,
-                             entry=task.serialize(),
+                             task_dict_string=task.serialize(),
                              created=datetime.datetime.now(),
                              modified=datetime.datetime.now(),
-                            ).execute()
+                            ).execute(database=None)
         except (pymysql.err.IntegrityError, peewee.IntegrityError) as e:
             log("task already inserted, reasserting the queue to",self.queue)
 
@@ -501,7 +598,7 @@ class Queue:
                                 queue=self.queue,
                             ).where(
                                 TaskEntry.key == task.key,
-                            ).execute()
+                            ).execute(database=None)
 
 
     def find_dependecies_states(self,task):
@@ -511,10 +608,10 @@ class Queue:
         log("find_dependecies_states for",task.key)
 
         dependencies=[]
-        for i_dep,dependency in enumerate(task.depends_on):
+        for i_dep, dependency in enumerate(task.depends_on):
             dependency_task=Task(dependency)
 
-            print(("task",task.key,"depends on task",dependency_task.key,i_dep,"/",len(task.depends_on)))
+            print(("task",task.key,"depends on task", dependency_task.key, i_dep, "/", len(task.depends_on)))
             dependency_instances=self.find_task_instances(dependency_task)
             print(("task instances for",dependency_task.key,len(dependency_instances)))
 
@@ -551,17 +648,32 @@ class Queue:
 
 
 
-    def task_locked(self,depends_on):
+    def task_locked(self, depends_on: List[types.TaskDict]):
         ""
+        if not isinstance(depends_on, list):
+            raise Exception(f"depends_on has unknown type {depends_on.__class__}, expected list")
+
         log("locking task",self.current_task)
-        self.log_task("task to lock...",state="locked")
+        self.log_task(f"task to lock by {len(depends_on)} dependencies",state="locked")
         if self.current_task is None:
             raise Exception("task must be available to lock")
 
-        self.current_task.depends_on=depends_on
+        self.current_task.depends_on = [] 
+        
+        for dependency in depends_on:
+            dependency_key = Task(dependency).key
+
+            #dependency_task = self.task_by_key(dependency_key)
+
+            dependency_reference = dict(
+                        task_key=dependency_key,
+                    )
+
+            self.current_task.depends_on.append(dependency_reference)
+
         serialized=self.current_task.serialize()
 
-        self.log_task("task to lock: serialized to %i"%len(serialized),state="locked")
+        self.log_task("task to lock: serialized to %i"%len(serialized), state="locked")
 
         n_tries_left=10
         retry_delay=2
@@ -570,13 +682,6 @@ class Queue:
 
                 self.move_task('running', 'locked', self.current_task, update_entry=True)
 
-               # r=TaskEntry.update({
-               #             TaskEntry.state:"locked",
-               #             TaskEntry.entry:serialized,
-               #         }).where(
-               #             TaskEntry.key==self.current_task.key,
-               #             TaskEntry.state=="running",
-               #          ).execute()
             except Exception as e:
                 log('failed to lock:',repr(e))
                 self.log_task("task to failed lock: %s; serialized to %i"%(repr(e),len(serialized)),state="failed_to_lock")
@@ -601,16 +706,16 @@ class Queue:
 
         r=TaskEntry.update({
                     TaskEntry.state:"done",
-                    TaskEntry.entry:self.current_task.serialize(),
+                    TaskEntry.task_dict_string:self.current_task.serialize(), # TODO this modifies serialization!
                     TaskEntry.modified:datetime.datetime.now(),
-                }).where(TaskEntry.key==self.current_task.key).execute()
+                }).where(TaskEntry.key==self.current_task.key).execute(database=None)
 
         if self.current_task_stored_key != self.current_task.key:
             r=TaskEntry.update({
                         TaskEntry.state:"done",
-                        TaskEntry.entry:self.current_task.serialize(),
+                        TaskEntry.task_dict_string:self.current_task.serialize(),
                         TaskEntry.modified:datetime.datetime.now(),
-                    }).where(TaskEntry.key==self.current_task_stored_key).execute()
+                    }).where(TaskEntry.key==self.current_task_stored_key).execute(database=None)
 
 
         self.current_task_status="done"
@@ -628,7 +733,7 @@ class Queue:
 
         self.log_task("task failed",self.current_task,"failed")
         
-        history=[model_to_dict(en) for en in TaskHistory.select().where(TaskHistory.key==task.key).order_by(TaskHistory.id.desc()).execute()]
+        history=[model_to_dict(en) for en in EventLog.select().where(EventLog.task_key==task.key).order_by(EventLog.id.desc()).execute(database=None)]
         n_failed = len([he for he in history if he['state'] == "failed"])
 
         self.log_task("task failed %i times already"%n_failed,task,"failed")
@@ -642,9 +747,9 @@ class Queue:
 
         r=TaskEntry.update({
                     TaskEntry.state:next_state,
-                    TaskEntry.entry:self.current_task.serialize(),
+                    TaskEntry.task_dict_string:self.current_task.serialize(),
                     TaskEntry.modified:datetime.datetime.now(),
-                }).where(TaskEntry.key==self.current_task.key).execute()
+                }).where(TaskEntry.key==self.current_task.key).execute(database=None)
 
         self.current_task_status = next_state
         self.current_task = None
@@ -652,16 +757,16 @@ class Queue:
 
     def wipe(self,wipe_from=["waiting"]):
         for fromk in wipe_from:
-            for key in self.list(fromk):
+            for key in self.list_tasks(fromk):
                 log("removing",fromk + "/" + key)
-                TaskEntry.delete().where(TaskEntry.key==key).execute()
+                TaskEntry.delete().where(TaskEntry.key==key).execute(database=None)
         
     @property
     def info(self):
         ""
         r={}
         for kind in "waiting","running","done","failed","locked":
-            r[kind]=len(self.list(kind))
+            r[kind]=len(self.list_tasks(kind))
         return r
 
     def show(self):
@@ -690,6 +795,30 @@ class Queue:
             pid=os.getpid(),
         )
         return "{fqdn}.{pid}".format(**d)
+    
+
+    def view_log(self, task_key=None, since=0):
+        c = EventLog.id >= since
+
+        if task_key is not None:
+            c &= EventLog.task_key==task_key
+
+        history=[ model_to_dict(en) for en in EventLog.select()\
+                                                      .where(c)\
+                                                      .order_by(EventLog.id.asc())\
+                                                      .execute(database=None) ]
+
+        return history
+    
+    def log_queue(self, message, spent_s, worker_id):
+        ""
+        return EventLog.insert(
+                             queue=self.queue,
+                             worker_id=self.worker_id,
+                             timestamp=datetime.datetime.now(),
+                             message=message,
+                             spent_s=spent_s,
+                        ).execute(database=None)
 
 
     def log_task(self, message, task=None, state=None, task_key=None):
@@ -706,27 +835,46 @@ class Queue:
         if state is None:
             state="undefined"
 
-        return TaskHistory.insert(
+        return EventLog.insert(
                              queue=self.queue,
-                             key=task_key,
+                             task_key=task_key,
                              state=state,
                              worker_id=self.worker_id,
                              timestamp=datetime.datetime.now(),
                              message=message,
-                        ).execute()
+                        ).execute(database=None)
 
-    def list(self,kind=None,kinds=None,fullpath=False):
+
+    def list(self, *args, **kwargs): # compatibility
+        logger.warning("please use list_tasks instead")
+        return self.list_tasks(*args, **kwargs)
+
+    def list_tasks(self, state=None, states=None, kind=None, kinds=None, decode=False):
         ""
-        if kinds is None:
-            kinds=["waiting"]
+
         if kind is not None:
-            kinds=[kind]
+            logger.warning("please use 'state' instead of 'kind' in list_tasks")
+            state = kind
 
-        kind_jobs = []
+        if state is not None:
+            states=[state]
 
-        for kind in kinds:
-            for task_entry in TaskEntry.select().where(TaskEntry.state==kind, TaskEntry.queue==self.queue):
-                kind_jobs.append(task_entry.key)
-        return kind_jobs
+        if kinds is not None:
+            logger.warning("please use 'states' instead of 'kinds' in list_tasks")
+            states = kinds
+
+        if states is None:
+            states=["waiting"]
+
+
+        jobs = []
+
+        logger.info("requested to list_tasks; queue: %s states: %s", self.queue, states)
+
+        for state in states:
+            for task_entry in TaskEntry.select().where(TaskEntry.state==state, TaskEntry.queue==self.queue):
+                jobs.append(task_entry.key)
+
+        return jobs
 
 
