@@ -1,10 +1,9 @@
-import yaml
 import datetime
 import os
 import time
 import socket
 from hashlib import sha224
-from collections import OrderedDict, defaultdict
+from collections import defaultdict
 import glob
 import logging
 from io import StringIO
@@ -12,19 +11,22 @@ import re
 import click
 import urllib.parse as urlparse# type: ignore
 
-from .core import Queue, Empty, Task, CurrentTaskUnfinished, TaskEntry
+from dqueue.core import Queue, Empty, Task, CurrentTaskUnfinished
+import dqueue.core as core
+import dqueue.typing as types
+from dqueue import tools
 
 from bravado.client import SwaggerClient
 
 
 
 class QueueProxy(Queue):
-    master = None
+    leader = None
     queue = None
     token = ""
 
     def __repr__(self):
-        return f"[ {self.__class__.__name__}: {self.master}@{self.queue} ]"
+        return f"[ {self.__class__.__name__}: {self.leader}@{self.queue} ]"
 
     def __init__(self, queue_uri="http://localhost:5000@default"):
         super().__init__()
@@ -33,17 +35,17 @@ class QueueProxy(Queue):
         if not r:
             raise Exception("uri does not match queue")
 
-        self.master = r.groups()[0]
+        self.leader = r.groups()[0]
         self.queue = r.groups()[1]
 
     def list_queues(self, pattern):
         print(self.client.queues.list().response().result)
-        return [QueueProxy(self.master+"@"+q) for q in self.client.queues.list().response().result]
+        return [QueueProxy(self.leader+"@"+q) for q in self.client.queues.list().response().result]
 
     @property
     def client(self):
         if getattr(self, '_client', None) is None:
-            self._client = SwaggerClient.from_url(self.master.strip("/")+"/apispec_1.json", config={'use_models': False})
+            self._client = SwaggerClient.from_url(self.leader.strip("/")+"/apispec_1.json", config={'use_models': False})
         return self._client
 
     def find_task_instances(self,task,klist=None):
@@ -53,19 +55,48 @@ class QueueProxy(Queue):
     def select_task_entry(self,key):
         raise NotImplementedError
     
+    def task_info(self, key):
+        return self.client.task.task_info(task_key=key).response().result
+
+    def task_by_key(self, key: str, decode: bool=False) -> types.TaskDict:
+        r = self.client.task.task_info(task_key=key).response().result
+
+        if decode:
+            r['task_dict'] = tools.decode_entry_data(r)
+
+        return r
+    
+    def view_log(self, task_key=None, since=0):
+        if task_key is None:
+            task_key = ""
+
+        return self.client.task.view_log(task_key=task_key,
+                                         since=since,
+                                         token=self.token).response().result
+    
+    def log_queue(self, message, spent_s=0):
+        self.logger.info("log queue %s", message)
+
+        return self.client.worker.logQueue(message=message,
+                                   spent_s=spent_s,
+                                   worker_id=self.worker_id,
+                                   token=self.token).response().result
+    
     def log_task(self, message, task=None, state=None, task_key=None):
+        self.logger.info("log_task %s", message)
+
         if task_key is None:
             if task is None:
                 task = self.current_task
 
             task_key = task.key
 
-        self.client.worker.logTask(message=message, 
+        return self.client.worker.logTask(message=message, 
                                    task_key=task_key, 
                                    state=state, 
                                    queue=self.queue, 
                                    worker_id=self.worker_id,
-                                   token=self.token)
+                                   token=self.token).response().result
 
     def insert_task_entry(self,task,state):
         raise NotImplementedError
@@ -92,7 +123,7 @@ class QueueProxy(Queue):
         if r.result is None:
             raise Empty()
 
-        self.current_task = Task.from_entry(r.result)
+        self.current_task = Task.from_task_dict(r.result)
         self.current_task_stored_key = self.current_task.key
 
         return self.current_task
@@ -122,17 +153,16 @@ class QueueProxy(Queue):
 
     def wipe(self,wipe_from=["waiting"]):
         #for fromk in wipe_from:
-        for key in self.list():
+        for key in self.list_tasks():
             self.logger.info("removing %s", key)
-            TaskEntry.delete().where(TaskEntry.key==key).execute()
+            core.TaskEntry.delete().where(core.TaskEntry.key==key).execute(database=None)
         
     def purge(self):
         nentries = self.client.tasks.purge().response().result
         self.logger.info("deleted %s", nentries)
 
 
-    def list(self):
-        print(dir(self.client.tasks))
+    def list_tasks(self):
         l = [task for task in self.client.tasks.listTasks().response().result['tasks']]
         self.logger.info(f"found tasks: {len(l)}")
         return l
@@ -140,7 +170,7 @@ class QueueProxy(Queue):
     @property
     def info(self):
         r={}
-        tasks = self.list()
+        tasks = self.list_tasks()
         for kind in "waiting","running","done","failed","locked":
             r[kind]=[t for t in tasks if t['state'] == kind]
         return r
@@ -152,3 +182,6 @@ class QueueProxy(Queue):
     def resubmit(self, scope, selector):
         return self.client.tasks.resubmit(scope=scope, selector=selector)
 
+    def try_all_locked(self):
+        return self.client.tasks.try_all_locked(worker_id=self.worker_id, token=self.token, queue=self.queue).response().result
+
